/*****************************************************************************
 *  Rohde&Schwarz Spectrum Analyzer instrument driver - K7S FM Stereo
 *  VXIpnp Style Instrument Driver
 *  Original Release: November 2009
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *****************************************************************************/

#include <math.h>
#include "rsspecan.h"

// Weigthing settings,`                    CCITT,         CCIR_U, CCIR_W,     A_W
static ViBoolean weightingArr[5][4] = {
    {VI_FALSE, VI_FALSE, VI_FALSE, VI_FALSE}, //None
    {VI_TRUE, VI_FALSE, VI_FALSE, VI_FALSE}, //CCITT
    {VI_FALSE, VI_TRUE, VI_FALSE, VI_FALSE}, //CCIR Unweighting
    {VI_FALSE, VI_FALSE, VI_TRUE, VI_FALSE}, //CCIR Weighting
    {VI_FALSE, VI_FALSE, VI_FALSE, VI_TRUE}
}; //A Weighting


static ViInt32 fmsAFFilterHPassStateArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_HIGH_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_HIGH_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_HIGH_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_HIGH_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_HIGH_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_HIGH_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_HIGH_PASS_ENABLED
};

static ViInt32 fmsAFFilterHPassFreqArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_HIGH_PASS_FREQUENCY,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_HIGH_PASS_FREQUENCY,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_HIGH_PASS_FREQUENCY,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_HIGH_PASS_FREQUENCY,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_HIGH_PASS_FREQUENCY,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_HIGH_PASS_FREQUENCY,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_HIGH_PASS_FREQUENCY
};

static ViInt32 fmsAFFilterLPassStateArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_LOW_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_LOW_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_LOW_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_LOW_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_LOW_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_LOW_PASS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_LOW_PASS_ENABLED
};

static ViInt32 fmsAFFilterLPassFreqRelArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_LOW_PASS_FREQUENCY_RELATIVE
};

static ViInt32 fmsAFFilterLPassFreqAbsArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_LOW_PASS_FREQUENCY_ABSOLUTE
};

static ViInt32 fmsAFFilterCCITTArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_CCITT,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_CCITT,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_CCITT,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_CCITT,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_CCITT,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_CCITT,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_CCITT
};

static ViInt32 fmsAFFilterCCIRUnweightedArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_CCIR_UNWEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_CCIR_UNWEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_CCIR_UNWEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_CCIR_UNWEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_CCIR_UNWEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_CCIR_UNWEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_CCIR_UNWEIGHTED
};

static ViInt32 fmsAFFilterCCIRWeightedArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_CCIR_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_CCIR_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_CCIR_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_CCIR_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_CCIR_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_CCIR_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_CCIR_WEIGHTED
};

static ViInt32 fmsAFFilterAWeightedArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_A_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_A_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_A_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_A_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_A_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_A_WEIGHTED,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_A_WEIGHTED
};

static ViInt32 fmsAFFilterDeemStateArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_DEEMPHASIS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_DEEMPHASIS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_DEEMPHASIS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_DEEMPHASIS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_DEEMPHASIS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_DEEMPHASIS_ENABLED,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_DEEMPHASIS_ENABLED
};

static ViInt32 fmsAFFilterDeemTimeArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_DEEMPHASIS_TIME_CONSTANT,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_DEEMPHASIS_TIME_CONSTANT,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_DEEMPHASIS_TIME_CONSTANT,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_DEEMPHASIS_TIME_CONSTANT,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_DEEMPHASIS_TIME_CONSTANT,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_DEEMPHASIS_TIME_CONSTANT,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_DEEMPHASIS_TIME_CONSTANT
};

static ViInt32 fmsAFFilterCoupledArr[] = {
    RSSPECAN_ATTR_FM_STEREO_LEFT_AF_FILTER_COUPLED,
    RSSPECAN_ATTR_FM_STEREO_RIGHT_AF_FILTER_COUPLED,
    RSSPECAN_ATTR_FM_STEREO_MPX_AF_FILTER_COUPLED,
    RSSPECAN_ATTR_FM_STEREO_MONO_AF_FILTER_COUPLED,
    RSSPECAN_ATTR_FM_STEREO_STEREO_AF_FILTER_COUPLED,
    RSSPECAN_ATTR_FM_STEREO_RDS_AF_FILTER_COUPLED,
    RSSPECAN_ATTR_FM_STEREO_PILOT_AF_FILTER_COUPLED
};


/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/
/// HIFN This function selects the FM Stereo mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_FMStereoMode(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MODE, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches between Stereo FM and Spectrum mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control switches between Stereo FM and Spectrum mode.
ViStatus _VI_FUNC rsspecan_SetFMStereoState(ViSession instrSession,
                                            ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_ENABLED, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the FM Stereo trigger settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR triggerSource/Specifies the source of the trigger signal that causes the
/// HIPAR triggerSource/analyzer to leave the Wait-For-Trigger state.
/// HIPAR triggerLevel/Sets the level of the trigger source.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoTrigger(ViSession instrSession,
                                                    ViInt32   triggerSource,
                                                    ViReal64  triggerLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_TRIGGER_SOURCE, triggerSource),
            2, "Trigger Source");

    switch (triggerSource)
    {
    case RSSPECAN_VAL_FMS_TRG_IMM:
        break;
    case RSSPECAN_VAL_FMS_TRG_EXT:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_EXTERNAL_TRIGGER_LEVEL, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_IFP:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_IFP_LEVEL, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_TIME:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_TRIGGER_TIME_INTERVAL, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_LEFT:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_LEFT, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_RIGH:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_RIGHT, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_MPX:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_MPX, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_MONO:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_MONO, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_STER:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_STEREO, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_RDS:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_RDS, triggerLevel),
                3, "Trigger Level");
        break;
    case RSSPECAN_VAL_FMS_TRG_PIL:
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_TRIGGER_LEVEL_PILOT, triggerLevel),
                3, "Trigger Level");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, triggerSource), 2, "Trigger Source");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures phase noise marker measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR state/This control switches the phase noise measurement for the
/// HIPAR state/specified marker on or off in the specified window.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoPhaseNoiseMarker(ViSession instrSession,
                                                             ViInt32   window,
                                                             ViInt32   marker,
                                                             ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FM_STEREO_PHASE_NOISE_MARKER_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_w),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FM_STEREO_PHASE_NOISE_MARKER_ENABLED,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_m),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_w, repCap_m);

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_FM_STEREO_PHASE_NOISE_MARKER_ENABLED, state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures difference frequency distortion
/// HIFN measurement. This function is only available for AF spectrum
/// HIFN measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control activates difference frequency distortion measurement.
/// HIPAR unit/This control defines the unit for the difference frequency
/// HIPAR unit/distortion measurement results.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoDifferenceFrequencyDistortion(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state,
                            ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FM_STEREO_DIFFERENCE_FREQUENCY_DISTORTION_ENABLED,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_FM_STEREO_DIFFERENCE_FREQUENCY_DISTORTION_ENABLED, state),
            3, "State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_DIFFERENCE_FREQUENCY_DISTORTION_UNIT, unit),
            4, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures intermodulation distortion
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR state/This control activates intermodulation distortion measurement.
/// HIPAR unit/This control defines the unit for intermodulation distortion measurement
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoIntermodulationDistortion(
                            ViSession instrSession,
                            ViInt32   window,
                            ViBoolean state,
                            ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FM_STEREO_INTERMODULATION_DISTORTION_STATE,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, repCap, RSSPECAN_ATTR_FM_STEREO_INTERMODULATION_DISTORTION_STATE, state),
            2, "State");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_INTERMODULATION_DISTORTION_UNIT, unit),
            3, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates automatic scaling of the y-axis for FM
/// HIFN stereo measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR state/This control activates automatic scaling of the y-axis for FM
/// HIPAR state/stereo measurements.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFAutoScale(ViSession instrSession,
                                                        ViBoolean state)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_AF_AUTO_SCALE, state),
            2, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the evaluation method of the measured
/// HIFN data that is to be displayed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR displayConfiguration/This control selects the evaluation method of the measured data
/// HIPAR displayConfiguration/that is to be displayed.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoDisplayConfig(ViSession instrSession,
                                                          ViInt32   window,
                                                          ViInt32   displayConfiguration)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FMS_DISPLAY_CONFIG,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, repCap, RSSPECAN_ATTR_FMS_DISPLAY_CONFIG, displayConfiguration),
            3, "Display Configuration");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the reference deviation required for
/// HIFN relative deviation measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR referenceDeviation/This control defines the reference deviation required for
/// HIPAR referenceDeviation/relative deviation measurements.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoReferenceDeviation(
                            ViSession instrSession,
                            ViReal64  referenceDeviation)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_REFERENCE_DEVIATION, referenceDeviation),
            2, "Reference Deviation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures result summary display.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelType/This control select the channel type.
/// HIPAR detector/This control defines the detector used to determine the
/// HIPAR detector/deviation value of the selected channel of the FM stereo signal
/// HIPAR detector/in the result summary.
/// HIPAR mode/This control defines the result summary mode for the absolute
/// HIPAR mode/and relative deviation. It does not affect the trace mode.
/// HIPAR coupled/This control couples the channel settings to other channels,
/// HIPAR coupled/i.e. channels for which this control is set to "ON" are
/// HIPAR coupled/configured identically.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoResultSummarySetup(
                            ViSession instrSession,
                            ViInt32   channelType,
                            ViInt32   detector,
                            ViInt32   mode,
                            ViBoolean coupled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (channelType)
    {
    case RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_LEFT_RESULT_SUMMARY_DETECTOR, detector),
                3, "Detector");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_LEFT_RESULT_SUMMARY_MODE, mode),
                4, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_LEFT_RESULT_SUMMARY_COUPLED, coupled),
                5, "Coupled");
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_RIGHT_RESULT_SUMMARY_DETECTOR, detector),
                3, "Detector");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_RIGHT_RESULT_SUMMARY_MODE, mode),
                4, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_RIGHT_RESULT_SUMMARY_COUPLED, coupled),
                5, "Coupled");
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_MPX:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MPX_RESULT_SUMMARY_DETECTOR, detector),
                3, "Detector");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MPX_RESULT_SUMMARY_MODE, mode),
                4, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MPX_RESULT_SUMMARY_COUPLED, coupled),
                5, "Coupled");
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_MONO:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MONO_RESULT_SUMMARY_DETECTOR, detector),
                3, "Detector");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MONO_RESULT_SUMMARY_MODE, mode),
                4, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MONO_RESULT_SUMMARY_COUPLED, coupled),
                5, "Coupled");
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_STER:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_STEREO_RESULT_SUMMARY_DETECTOR, detector),
                3, "Detector");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_STEREO_RESULT_SUMMARY_MODE, mode),
                4, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_STEREO_RESULT_SUMMARY_COUPLED, coupled),
                5, "Coupled");
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_RDS:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_RDS_RESULT_SUMMARY_DETECTOR, detector),
                3, "Detector");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_RDS_RESULT_SUMMARY_MODE, mode),
                4, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_RDS_RESULT_SUMMARY_COUPLED, coupled),
                5, "Coupled");
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT:
        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_PILOT_RESULT_SUMMARY_DETECTOR, detector),
                3, "Detector");

        viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_PILOT_RESULT_SUMMARY_MODE, mode),
                4, "Mode");

        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_PILOT_RESULT_SUMMARY_COUPLED, coupled),
                5, "Coupled");
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 2, "Channel Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function determines the reference deviation from the
/// HIFN current channel measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelType/This control select the channel type.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoMeasToRef(ViSession instrSession,
                                                      ViInt32   channelType)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    switch (channelType)
    {
    case RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_LEFT_MEAS_TO_REF, NULL));
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_RIGHT_MEAS_TO_REF, NULL));
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_MPX:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MPX_MEAS_TO_REF, NULL));
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_MONO:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_MONO_MEAS_TO_REF, NULL));
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_STER:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_STEREO_MEAS_TO_REF, NULL));
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_RDS:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_RDS_MEAS_TO_REF, NULL));

        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT:
        checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_PILOT_MEAS_TO_REF, NULL));
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 2, "Channel Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures analog demodulator measuring time
/// HIFN parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR measuringTime/This control sets the measuring time for the analog
/// HIPAR measuringTime/Demodulation.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoMeasurementTime(ViSession instrSession,
                                                            ViReal64  measuringTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_MEAS_TIME, measuringTime),
            2, "Measuring Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures different filter settings for each channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelType/This control select the channel type.
/// HIPAR highPass/This control selects the high pass filter type for the specified
/// HIPAR highPass/channel type.
/// HIPAR lowPass/This control selects the low pass filter type for the specified
/// HIPAR lowPass/channel type.
/// HIPAR weighting/This control activates/deactivates the weighted filter for the
/// HIPAR weighting/specified channel type.
/// HIPAR deemphasis/This control selects the deemphasis for the specified channel
/// HIPAR deemphasis/type.
/// HIPAR coupled/This control couples the channel settings to other channels,
/// HIPAR coupled/i.e. channels for which this control is set to "ON" are
/// HIPAR coupled/configured identically.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFFilter(ViSession instrSession,
                                                     ViInt32   channelType,
                                                     ViInt32   highPass,
                                                     ViInt32   lowPass,
                                                     ViInt32   weighting,
                                                     ViInt32   deemphasis,
                                                     ViBoolean coupled)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelType, RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT, RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT),
            2, "Channel Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, highPass, RSSPECAN_VAL_FMS_HPAS_FILT_NONE, RSSPECAN_VAL_FMS_HPAS_FILT_300HZ),
            3, "High Pass");
    if ((RsCore_InvalidViInt32Range(instrSession, lowPass, RSSPECAN_VAL_FMS_LOWPAS_FILT_NONE,
                                    RSSPECAN_VAL_FMS_LOWPAS_FILT_25PCT) == RS_ERROR_INVALID_PARAMETER) &&
        (RsCore_InvalidViInt32Range(instrSession, lowPass, RSSPECAN_VAL_FMS_LOWPAS_FILT_3KHZ,
                                    RSSPECAN_VAL_FMS_LOWPAS_FILT_150KHZ) == RS_ERROR_INVALID_PARAMETER))
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, lowPass), 4, "Low Pass");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, weighting, RSSPECAN_VAL_FMS_WEIGHTING_NONE, RSSPECAN_VAL_FMS_WEIGHTING_A),
            5, "Weighting");

    if (weighting == RSSPECAN_VAL_FMS_WEIGHTING_NONE)
    {
        if (highPass == RSSPECAN_VAL_FMS_HPAS_FILT_NONE)
        {
            viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "",
                            fmsAFFilterHPassStateArr[channelType],
                            VI_FALSE), 3, "High Pass")
        }
        else
        {
            viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterHPassStateArr[channelType], VI_TRUE),
                    3, "High Pass");

            viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "",
                            fmsAFFilterHPassFreqArr[channelType],
                            rsspecan_rngHpasFilterFreq.entries[highPass-1]), 3, "High Pass");
        }

        if (lowPass == RSSPECAN_VAL_FMS_LOWPAS_FILT_NONE)
        {
            viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "",
                            fmsAFFilterLPassStateArr[channelType],
                            VI_FALSE), 4, "Low Pass")
        }
        else
        {
            viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterLPassStateArr[channelType], VI_TRUE),
                    4, "Low Pass");

            if (lowPass < RSSPECAN_VAL_FMS_LOWPAS_FILT_3KHZ)
                viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "",
                            fmsAFFilterLPassFreqRelArr[channelType],
                            rsspecan_rngFMSLPassFilterFrequencyRel.entries[lowPass-1]), 4, "Low Pass")
            else
                viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "",
                            fmsAFFilterLPassFreqAbsArr[channelType],
                            rsspecan_rngFMSLPassFilterFrequency.entries[lowPass-10]), 4, "Low Pass");
        }
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterCCITTArr[channelType], weightingArr[weighting][0]),
            5, "Weighting");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterCCIRUnweightedArr[channelType], weightingArr[weighting][1]),
            5, "Weighting");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterCCIRWeightedArr[channelType], weightingArr[weighting][2]),
            5, "Weighting");

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterAWeightedArr[channelType], weightingArr[weighting][3]),
            5, "Weighting");

    if (deemphasis == RSSPECAN_VAL_FMS_DEEMPH_NONE)
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterDeemStateArr[channelType], VI_FALSE),
                    6, "Deemphasis")
    }
    else
    {
        viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterDeemStateArr[channelType], VI_TRUE),
                6, "Deemphasis");

        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "",fmsAFFilterDeemTimeArr[channelType],
                        rsspecan_rngFiltDempTimeConst.entries[deemphasis]), 6, "Deemphasis");
    }

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", fmsAFFilterCoupledArr[channelType], coupled),
            7, "Coupled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches all AF Filters off at once.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
ViStatus _VI_FUNC rsspecan_AllFMStereoAFFiltersOff(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rsspecan_SetAttributeViString(instrSession, "", RSSPECAN_ATTR_FM_STEREO_ALL_AF_FILTERS_OFF, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function configures deviation settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR deviation/Switches between logarithmic and linear display.
/// HIPAR devperDivision/Defines the deviation per division.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFRange(ViSession instrSession,
                                                    ViInt32   deviation,
                                                    ViReal64  devperDivision)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_VERTICAL_SCALE, deviation),
            2, "Deviation");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_DISP_Y_AXIS_SCALING, devperDivision),
            3, "Dev Per Division");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects between % and dB for displaying the THD
/// HIFN measurement result.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR unit/This control selects between % and dB for displaying the THD
/// HIPAR unit/measurement result.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFRangeTHDUnit(ViSession instrSession,
                                                           ViInt32   unit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_ANALOG_DEMOD_THD_UNIT, unit),
            2, "Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the deviation units.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR absoluteDevUnit/Selects the unit for absolute deviation measurements.
/// HIPAR relativeDevUnit/Selects the unit for relative deviation measurements.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoAFRangeDevUnit(ViSession instrSession,
                                                           ViInt32   absoluteDevUnit,
                                                           ViInt32   relativeDevUnit)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_AF_RANGE_DEV_UNIT_ABSOLUTE, absoluteDevUnit),
            2, "Absolute Dev Unit");

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_STEREO_AF_RANGE_DEV_UNIT_RELATIVE, relativeDevUnit),
            3, "Relative Dev Unit");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activate and configure the zoom function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR zoomState/Turns on or off the zoom function for the trace data of the FM
/// HIPAR zoomState/demodulator.
/// HIPAR startTime/Selects the start time for the display of the single measured
/// HIPAR startTime/values of the FM demodulator.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoTimeDomainZoom(ViSession instrSession,
                                                           ViBoolean zoomState,
                                                           ViReal64  startTime)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_ENABLED, zoomState),
            2, "Zoom State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_START, startTime),
            3, "Start Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function configures the zoom length settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR zoomLengthMode/Defines whether the length of the zoom area for the
/// HIPAR zoomLengthMode/analog-demodulated measurement data is defined automatically or
/// HIPAR zoomLengthMode/manually.
/// HIPAR zoomLength/Defines the length of the zoom area for the analog-demodulated
/// HIPAR zoomLength/measurement data manually.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoTimeDomainZoomLength(
                            ViSession instrSession,
                            ViInt32   zoomLengthMode,
                            ViReal64  zoomLength)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViInt32(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_LENGTH_MODE, zoomLengthMode),
            2, "Zoom Length Mode");

    if (zoomLengthMode == RSSPECAN_VAL_MAN)
    {
        viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_DEMOD_ZOOM_LENGTH, zoomLength),
                3, "Zoom Length");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function activates and configures the squelch function.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR squelchState/Activates the squelch function, i.e. if the signal falls below a
/// HIPAR squelchState/defined threshold (Squelch Level), the demodulated data is
/// HIPAR squelchState/automatically set to 0.
/// HIPAR squelchLevel/Defines the level threshold below which the demodulated data is
/// HIPAR squelchLevel/set to 0 if squelching is enabled.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoSquelch(ViSession instrSession,
                                                    ViBoolean squelchState,
                                                    ViReal64  squelchLevel)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_SetAttributeViBoolean(instrSession, "", RSSPECAN_ATTR_FM_STEREO_SQUELCH, squelchState),
            2, "Squelch State");

    viCheckParm(rsspecan_SetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_FM_STEREO_SQUELCH_LEVEL, squelchLevel),
            3, "Squelch Level");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query FM Stereo Carrier Frequency
 * Purpose:  This function queries the carrier frequency of the stereo
 *           measurement.
 *****************************************************************************/
/// HIFN This function queries the carrier frequency of the stereo
/// HIFN measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR carrierFrequency/Returns the carrier frequency of the stereo measurement.
ViStatus _VI_FUNC rsspecan_QueryFMStereoCarrierFrequency(ViSession instrSession,
                                                         ViReal64* carrierFrequency)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, "", RSSPECAN_ATTR_QUERY_FM_STEREO_CARRIER_FREQUENCY, carrierFrequency),
            2, "Carrier Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function selects the trace modes of the FM stereo to be
/// HIFN measured simultaneously. For each trace a mode can be defined,
/// HIFN however only if the specified channel is currently displayed in
/// HIFN one of the four screens. If a trace mode is set for a channel
/// HIFN that is not displayed, a query error is generated.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelType/This control select the channel type.
/// HIPAR measurementType/This parameter selects the measurement.
/// HIPAR arraySize/Sets the number of items in Trace Modes array.
/// HIPAR traceModes/Selects the trace modes.
ViStatus _VI_FUNC rsspecan_ConfigureFMStereoTraceModes(ViSession instrSession,
                                                       ViInt32   channelType,
                                                       ViInt32   measurementType,
                                                       ViInt32   arraySize,
                                                       ViInt32   traceModes[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  i = 0;
    ViChar*  pbuffer = cmd;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7S"));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, channelType, RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT, RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT),
            2, "Channel Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurementType, RSSPECAN_VAL_FMS_AFSP, RSSPECAN_VAL_FMS_TDOM),
            3, "Measurement Type");
    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, arraySize, 1),
                4, "Array Size");

    for (i = 0; i < arraySize; i++)
        viCheckParm(RsCore_InvalidViInt32Range(instrSession, traceModes[i], RSSPECAN_VAL_FMS_TRAC_MOD_WRITE, RSSPECAN_VAL_FMS_TRAC_MOD_OFF),
                5, "Trace Modes");

    pbuffer += snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:SFM:%s:%s:TYPE ",
                        rsspecan_rngFMSChanType.entries[channelType].cmdString,
                        (ViString)rsspecan_rngFMSMeasType.entries[measurementType].cmdString);

    for (i = 0; i < arraySize; i++)
        pbuffer += sprintf(pbuffer, "%s,", rsspecan_rngTraceMode.entries[traceModes[i]].cmdString);

    pbuffer--;
    *pbuffer = '\0';

    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: FM Stereo Difference Frequency Distortion Search Signal
 * Purpose:  This function starts the search of the signals required for the
 *           difference frequency distortion measurement.
 *****************************************************************************/
/// HIFN This function starts the search of the signals required for the
/// HIFN difference frequency distortion measurement..
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/Sets the timeout for signal search.
ViStatus _VI_FUNC rsspecan_FMStereoDifferenceFrequencyDistortionSearchSignal(
                            ViSession instrSession,
                            ViInt32   window,
                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FM_STEREO_DIFFERENCE_FREQUENCY_DISTORTION_SEARCH_SIGNAL,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 600000UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_FM_STEREO_DIFFERENCE_FREQUENCY_DISTORTION_SEARCH_SIGNAL, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function starts the search of the signals required for the
/// HIFN intermodulation distortion measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR timeout/Sets the timeout for signal search.
ViStatus _VI_FUNC rsspecan_FMStereoIntermodulationDistortionSearchSignal(
                            ViSession instrSession,
                            ViInt32   window,
                            ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViUInt32 old_timeout = 0;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_FM_STEREO_INTERMODULATION_DISTORTION_SEARCH_SIGNAL,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 600000UL), 3, "Timeout");

    checkErr(rsspecan_GetTimeout(instrSession, &old_timeout));
    checkErr(rsspecan_SetTimeout(instrSession, timeout));

    checkErr(rsspecan_SetAttributeViString(instrSession, repCap, RSSPECAN_ATTR_FM_STEREO_INTERMODULATION_DISTORTION_SEARCH_SIGNAL, NULL));

Error:
    if (old_timeout > 0)
        rsspecan_SetTimeout(instrSession, old_timeout);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the result of the phase noise measurement
/// HIFN in the selected measurement window. The measurement will be
/// HIFN switched on, if necessary.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR marker/This control selects marker.
/// HIPAR phaseNoiseResult/Returns the result of the phase-noise measurement in the
/// HIPAR phaseNoiseResult/selected measurement window. The measurement will be switched
/// HIPAR phaseNoiseResult/on, if necessary.
ViStatus _VI_FUNC rsspecan_QueryFMStereoPhaseNoiseResult(ViSession instrSession,
                                                         ViInt32   window,
                                                         ViInt32   marker,
                                                         ViReal64* phaseNoiseResult)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_m[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_w[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_QUERY_FM_STEREO_PHASE_NOISE_RESULT,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap_w),
            2, "Window");
    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_QUERY_FM_STEREO_PHASE_NOISE_RESULT,
            "Marker", marker - 1, RS_REPCAP_BUF_SIZE, repCap_m),
            3, "Marker");

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,%s", repCap_w, repCap_m);

    viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_PHASE_NOISE_RESULT, phaseNoiseResult),
            4, "Phase Noise Result");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the result of the difference frequency
/// HIFN distortion measurement in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR differenceFrequencyDistortion/Returns the difference frequency distortion factors of 2nd and
/// HIPAR differenceFrequencyDistortion/3rd order.
ViStatus _VI_FUNC rsspecan_QueryFMStereoDifferenceFrequencyDistortionResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  differenceFrequencyDistortion[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7S"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:FUNC:DFD:RES?", window);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 10, differenceFrequencyDistortion, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the result of the intermodulation
/// HIFN distortion measurement in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR intermodulationDistortion/Returns the intermodulation distortion factors of 2nd and 3rd
/// HIPAR intermodulationDistortion/order.
ViStatus _VI_FUNC rsspecan_QueryFMStereoIntermodulationDistortionResult(
                            ViSession instrSession,
                            ViInt32   window,
                            ViReal64  intermodulationDistortion[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7S"));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:FUNC:IMD:RES?", window);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 10, intermodulationDistortion, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN This function queries the results of the stereo measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR absoluteDeviation/Returns the absolute deviation values for all channels.
/// HIPAR relativeDeviation/Returns the relative deviation values for all channels.
/// HIPAR sinad/Returns the Signal-to-noise-and-distortion values for all channels.
/// HIPAR thd/Returns the total harmonic distortion values for all channels.
/// HIPAR modulationFrequency/Returns the modulation frequency values for all channels.
ViStatus _VI_FUNC rsspecan_QueryFMStereoMeasurementResults(ViSession instrSession,
                                                           ViReal64  absoluteDeviation[],
                                                           ViReal64  relativeDeviation[],
                                                           ViReal64  sinad[],
                                                           ViReal64  thd[],
                                                           ViReal64  modulationFrequency[])
{
    ViStatus  error = VI_SUCCESS;
    ViAReal64 data = NULL;
    ViInt32   i = 0;
    ViInt32   arraySize = 0;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7S"));

    checkErr(RsCore_QueryFloatArray(instrSession, "CALC:MARK:FUNC:SFM:RES? SUMM", &data, &arraySize));

    arraySize = arraySize / 5;
    for (i = 0; i < arraySize; i++)
    {
        absoluteDeviation[i] = data[i * 5 + 0];
        relativeDeviation[i] = data[i * 5 + 1];
        sinad[i] = data[i * 5 + 2];
        thd[i] = data[i * 5 + 3];
        modulationFrequency[i] = data[i * 5 + 4];
    }

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    if (data) free(data);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the results of the measurement type for
/// HIFN the selected channel in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channelType/Document parameter here.
/// HIPAR measurementType/Document parameter here.
/// HIPAR result/Document parameter here.
ViStatus _VI_FUNC rsspecan_QueryFMStereoChannelTypeResults(ViSession instrSession,
                                                           ViInt32   window,
                                                           ViInt32   channelType,
                                                           ViInt32   measurementType,
                                                           ViReal64* result)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_GetAttributeRepCapName(instrSession, RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_ADEV,
            "Window", window , RS_REPCAP_BUF_SIZE, repCap),
            2, "Window");
    switch (channelType)
    {
    case RSSPECAN_VAL_FSM_CHAN_TYPE_LEFT:
        switch (measurementType)
        {
        case RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_ADEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_RDEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_SINAD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_THD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_THD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_AFR:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_AFR, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FSM_MEAS_TYPE_XTALK:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_LEFT_RESULT_XTALK, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 3, "Measurement Type");
        }
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_RIGHT:
        switch (measurementType)
        {
        case RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_ADEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_RDEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_SINAD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_THD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_THD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_AFR:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_AFR, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FSM_MEAS_TYPE_XTALK:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RIGHT_RESULT_XTALK, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 3, "Measurement Type");
        }
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_MPX:
        switch (measurementType)
        {
        case RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_ADEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_RDEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_SINAD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_THD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_THD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_AFR:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_AFR, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FSM_MEAS_TYPE_XTALK:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MPX_RESULT_XTALK, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 3, "Measurement Type");
        }
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_MONO:
        switch (measurementType)
        {
        case RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_ADEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_RDEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_SINAD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_THD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_THD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_AFR:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_AFR, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FSM_MEAS_TYPE_XTALK:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_MONO_RESULT_XTALK, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 3, "Measurement Type");
        }
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_STER:
        switch (measurementType)
        {
        case RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_ADEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_RDEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_SINAD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_THD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_THD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_AFR:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_AFR, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FSM_MEAS_TYPE_XTALK:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_STEREO_RESULT_XTALK, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 3, "Measurement Type");
        }
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_RDS:
        switch (measurementType)
        {
        case RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_ADEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_RDEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_SINAD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_THD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_THD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_AFR:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_AFR, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FSM_MEAS_TYPE_XTALK:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_RDS_RESULT_XTALK, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 3, "Measurement Type");
        }
        break;
    case RSSPECAN_VAL_FSM_CHAN_TYPE_PILOT:
        switch (measurementType)
        {
        case RSSPECAN_VAL_FMS_MEAS_TYPE_ADEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_ADEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_RDEV:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_RDEV, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_SINAD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_SINAD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_THD:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_THD, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FMS_MEAS_TYPE_AFR:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_AFR, result),
                    5, "Result");
            break;
        case RSSPECAN_VAL_FSM_MEAS_TYPE_XTALK:
            viCheckParm(rsspecan_GetAttributeViReal64(instrSession, repCap, RSSPECAN_ATTR_QUERY_FM_STEREO_PILOT_RESULT_XTALK, result),
                    5, "Result");
            break;
        default:
            viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 3, "Measurement Type");
        }
        break;
    default:
        viCheckParm(RsCore_InvalidViInt32Value(instrSession, channelType), 2, "Channel Type");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries all available measurement values for the
/// HIFN selected channel in the specified window.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR window/This control selects the measurement window.
/// HIPAR channelType/This control select the channel type.
/// HIPAR result/Returns all available measurement values for the selected
/// HIPAR result/channel in the specified window.
ViStatus _VI_FUNC rsspecan_QueryFMStereoChannelTypeAllResults(
                            ViSession instrSession,
                            ViInt32   window,
                            ViInt32   channelType,
                            ViReal64  result[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2string;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, window, 1, 4),
            2, "Window");

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7S"));
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngFMSChanType, channelType, &p2string),
                2, "Channel Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC%ld:MARK:FUNC:SFM:%s:RES? ALL", window, p2string);
    checkErr(RsCore_QueryFloatArrayToUserBuffer(instrSession, cmd, 65535, result, NULL));

    checkErr(rsspecan_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function reads the result data of the FM stereo signal in
/// HIFN the specified trace mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR channelType/This control select the channel type.
/// HIPAR measurementType/This parameter selects the measurement.
/// HIPAR traceMode/Selects the trace mode to be measured in parallel by selected
/// HIPAR traceMode/demodulation.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR arraySize/This parameter determines the size of array result values. The
/// HIPAR arraySize/Result Values array must contain at least this number of items.
/// HIPAR resultValues/Returns the result values.
/// HIPAR returnedValues/Returns the number of returned values.
ViStatus _VI_FUNC rsspecan_QueryFMStereoResultValues(ViSession instrSession,
                                                     ViInt32   channelType,
                                                     ViInt32   measurementType,
                                                     ViInt32   traceMode,
                                                     ViUInt32  timeout,
                                                     ViInt32   arraySize,
                                                     ViReal64  resultValues[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar*  p2stringChannel;
    ViChar*  p2stringMeas;
    ViChar*  p2stringTrace;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_CheckInstrumentOptions(instrSession, "K7S"));
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngFMSChanType, channelType, &p2stringChannel),
                2, "Channel Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngFMSMeasType, measurementType, &p2stringMeas),
                3, "Measurement Type");
    viCheckParm(RsCore_FindRangeTableDiscreteCmdString(instrSession, &rsspecan_rngTraceMode, traceMode, &p2stringTrace),
                4, "Trace Mode");
    viCheckParm(RsCore_InvalidViUInt32Range(instrSession, timeout, 0, 4294967295UL), 5, "Timeout");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, ":FORM REAL,32;:SENS:SFM:%s:%s:RES? %s", p2stringChannel, p2stringMeas,
             p2stringTrace);
    checkErr(
        RsCore_QueryFloatArrayToUserBufferWithOpc(instrSession, cmd, timeout, arraySize, resultValues, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
